@@ -61,31 +61,21 @@ sub new {
     # Create email address object
     my $x0 = Sisimai::Address->parse( [ $argvs->{'addresser'} ] );
     my $y0 = Sisimai::Address->parse( [ $argvs->{'recipient'} ] );
-    my $v0 = undef;
     my @v1 = ();
 
-    if( ref $x0 eq 'ARRAY' ) {
-        $v0 = Sisimai::Address->new( shift @$x0 );
+    return undef unless ref $x0 eq 'ARRAY';
+    return undef unless ref $y0 eq 'ARRAY';
 
-        if( ref $v0 eq 'Sisimai::Address' ) {
-            $thing->{'addresser'} = $v0;
-            $thing->{'senderdomain'} = $v0->host;
-        }
-    }
-
-    if( ref $y0 eq 'ARRAY' ) {
-        $v0 = Sisimai::Address->new( shift @$y0 );
-
-        if( ref $v0 eq 'Sisimai::Address' ) {
-            $thing->{'recipient'} = $v0;
-            $thing->{'destination'} = $v0->host;
-            $thing->{'alias'} = $argvs->{'alias'};
-        }
-    }
-    return undef unless ref $thing->{'recipient'} eq 'Sisimai::Address';
+    $thing->{'addresser'} = Sisimai::Address->new( shift @$x0 );
     return undef unless ref $thing->{'addresser'} eq 'Sisimai::Address';
+    $thing->{'senderdomain'} = $thing->{'addresser'}->host;
 
-    $thing->{'token'} = Sisimai::String->token( 
+    $thing->{'recipient'} = Sisimai::Address->new( shift @$y0 );
+    return undef unless ref $thing->{'recipient'} eq 'Sisimai::Address';
+    $thing->{'destination'} = $thing->{'recipient'}->host;
+    $thing->{'alias'} = $argvs->{'alias'};
+
+    $thing->{'token'} = Sisimai::String->token(
                             $thing->{'addresser'}->address,
                             $thing->{'recipient'}->address,
                             $argvs->{'timestamp'} );
@@ -94,28 +84,13 @@ sub new {
     $thing->{'timestamp'} = localtime Sisimai::Time->new( $argvs->{'timestamp'} );
     $thing->{'timezoneoffset'} = $argvs->{'timezoneoffset'} // '+0000';
 
-    @v1 = ( 
+    @v1 = (
         'listid', 'subject', 'messageid', 'smtpagent', 'diagnosticcode',
         'diagnostictype', 'deliverystatus', 'reason', 'lhost', 'rhost', 
-        'smtpcommand', 'feedbacktype', 'action', 'softbounce',
+        'smtpcommand', 'feedbacktype', 'action', 'softbounce', 'replycode',
     );
     $thing->{ $_ } = $argvs->{ $_ } // '' for @v1;
-    if( $thing->{'action'} =~ m/\A(.+?) .+/ ) {
-        # Action: expanded (to multi-recipient alias)
-        $thing->{'action'} = $1;
-    }
-
-    $thing->{'replycode'} = Sisimai::SMTP::Reply->find( $argvs->{'diagnosticcode'} );
-
-    if( length $thing->{'deliverystatus'} && length $thing->{'replycode'} ) {
-        # Check both of the first digit of "deliverystatus" and "replycode"
-        my $d1 = substr( $thing->{'deliverystatus'}, 0, 1 );
-        my $r1 = substr( $thing->{'replycode'}, 0, 1 );
-
-        # The first digits did not match: 5.1.1 250
-        $thing->{'replycode'} = '' unless $d1 eq $r1;
-    }
-    $thing->{'softbounce'} = 1 if $thing->{'replycode'} =~ m/\A4/;
+    $thing->{'replycode'} ||= Sisimai::SMTP::Reply->find( $argvs->{'diagnosticcode'} );
 
     return bless( $thing, __PACKAGE__ );
 }
@@ -137,6 +112,7 @@ sub make {
     require Sisimai::SMTP;
     my $delivered1 = $argvs->{'delivered'} // 0;
     my $messageobj = $argvs->{'data'};
+    my $mailheader = $argvs->{'data'}->{'header'};
     my $rfc822data = $messageobj->rfc822;
     my $fieldorder = { 'recipient' => [], 'addresser' => [] };
     my $objectlist = [];
@@ -174,6 +150,7 @@ sub make {
             'alias'          => $e->{'alias'}        // '',
             'action'         => $e->{'action'}       // '',
             'reason'         => $e->{'reason'}       // '',
+            'replycode'      => $e->{'replycode'}    // '',
             'smtpagent'      => $e->{'agent'}        // '',
             'recipient'      => $e->{'recipient'}    // '',
             'softbounce'     => $e->{'softbounce'}   // '',
@@ -253,6 +230,14 @@ sub make {
         next unless $p->{'timestamp'};
 
         OTHER_TEXT_HEADERS: {
+            # Scan "Received:" header of the original message
+            if( scalar @{ $mailheader->{'received'} } ) {
+                # Get localhost and remote host name from Received header.
+                my $r0 = $mailheader->{'received'};
+                $e->{'lhost'} ||= shift @{ Sisimai::RFC5322->received( $r0->[0] ) };
+                $e->{'rhost'} ||= pop @{ Sisimai::RFC5322->received( $r0->[-1] ) };
+            }
+
             for my $v ( 'rhost', 'lhost' ) {
                 $p->{ $v } =~ y/[]()//d;    # Remove square brackets and curly brackets from the host variable
                 $p->{ $v } =~ s/\A.+=//;    # Remove string before "="
@@ -295,15 +280,41 @@ sub make {
                 # Cleanup the value of "Diagnostic-Code:" header
                 $p->{'diagnosticcode'} =~ s/[ \t]+$EndOfEmail//;
                 $p->{'diagnosticcode'} =~ s/\r\z//g;
+
                 my $v = Sisimai::SMTP::Status->find( $p->{'diagnosticcode'} );
                 if( $v =~ m/\A[45][.][1-9][.][1-9]\z/ ) {
                     # Use the DSN value in Diagnostic-Code:
                     $p->{'deliverystatus'} = $v;
                 }
+
+                if( $p->{'reason'} eq 'mailererror' ) {
+                    $p->{'diagnostictype'} ||= 'X-UNIX';
+
+                } else {
+                    unless( $p->{'reason'} =~ m/\A(?:feedback|vacation)\z/ ) {
+                        $p->{'diagnostictype'} ||= 'SMTP' 
+                    }
+                }
             }
 
             # Check the value of SMTP command
             $p->{'smtpcommand'} = '' unless $p->{'smtpcommand'} =~ $rxcommands;
+
+            if( $p->{'action'} ) {
+                if( $p->{'action'} =~ m/\A(.+?) .+/ ) {
+                    # Action: expanded (to multi-recipient alias)
+                    $p->{'action'} = $1;
+                }
+            } else {
+                if( $p->{'reason'} eq 'expired' ) {
+                    # Action: delayed
+                    $p->{'action'} = 'delayed';
+
+                } elsif( $p->{'deliverystatus'} =~ m/\A[45]/ ) {
+                    # Action: failed
+                    $p->{'action'} = 'failed';
+                }
+            }
         }
         $o = __PACKAGE__->new( %$p );
         next unless defined $o;
@@ -347,11 +358,19 @@ sub make {
                     $o->softbounce( Sisimai::SMTP->is_softbounce( $pdsv ) ) if $o->softbounce < 0;
                 }
             }
+
+            if( $o->replycode ) {
+                # Check both of the first digit of "deliverystatus" and "replycode"
+                my $d1 = substr( $o->deliverystatus, 0, 1 );
+                my $r1 = substr( $o->replycode, 0, 1 );
+                $o->replycode('') unless $d1 eq $r1;
+            }
+
         } else {
             # The value of reason is "vacation" or "feedback"
             $o->softbounce(-1);
+            $o->replycode('');
         }
-
         push @$objectlist, $o;
 
     } # End of for(LOOP_DELIVERY_STATUS)
