@@ -5,8 +5,8 @@ use strict;
 use warnings;
 
 my $Re0 = {
-    'x-mailer' => qr/\A[<]SMTP32 v[\d.]+[>]\z/,
-    'subject'  => qr/\AUndeliverable Mail\z/,
+    'x-mailer' => qr/\A[<]SMTP32 v[\d.]+[>][ ]*\z/,
+    'subject'  => qr/\AUndeliverable Mail[ ]*\z/,
 };
 my $Re1 = {
     'begin'  => qr/\A\z/,    # Blank line
@@ -14,7 +14,6 @@ my $Re1 = {
     'rfc822' => qr/\AOriginal message follows[.]\z/,
     'endof'  => qr/\A__END_OF_EMAIL_MESSAGE__\z/,
 };
-
 my $ReSMTP = {
     'conn' => qr{(?:
          SMTP[ ]connection[ ]failed,
@@ -26,7 +25,6 @@ my $ReSMTP = {
     'rcpt' => qr|Additional RCPT TO generated following response:|,
     'data' => qr|DATA command generated response:|,
 };
-
 my $ReFailure = {
     'hostunknown' => qr{
         Unknown[ ]host
@@ -49,10 +47,7 @@ my $ReFailure = {
         \ADelivery[ ]failed[ ]\d+[ ]attempts
     }x,
 };
-
 my $Indicators = __PACKAGE__->INDICATORS;
-my $LongFields = Sisimai::RFC5322->LONGFIELDS;
-my $RFC822Head = Sisimai::RFC5322->HEADERFIELDS;
 
 sub description { 'IPSWITCH IMail Server' }
 sub smtpagent   { 'IMailServer' }
@@ -83,11 +78,11 @@ sub scan {
     $match ||= 1 if( defined $mhead->{'x-mailer'} && $mhead->{'x-mailer'} =~ $Re0->{'x-mailer'} );
     return undef unless $match;
 
-    my $dscontents = []; push @$dscontents, __PACKAGE__->DELIVERYSTATUS;
+    my $dscontents = [ __PACKAGE__->DELIVERYSTATUS ];
     my @hasdivided = split( "\n", $$mbody );
-    my $rfc822next = { 'from' => 0, 'to' => 0, 'subject' => 0 };
     my $rfc822part = '';    # (String) message/rfc822-headers part
-    my $previousfn = '';    # (String) Previous field name
+    my $rfc822list = [];    # (Array) Each line in message/rfc822 part string
+    my $blanklines = 0;     # (Integer) The number of blank lines
     my $readcursor = 0;     # (Integer) Points the current cursor position
     my $recipients = 0;     # (Integer) The number of 'Final-Recipient' header
     my $v = undef;
@@ -112,27 +107,12 @@ sub scan {
 
         if( $readcursor & $Indicators->{'message-rfc822'} ) {
             # After "message/rfc822"
-            if( $e =~ m/\A([-0-9A-Za-z]+?)[:][ ]*.+\z/ ) {
-                # Get required headers only
-                my $lhs = lc $1;
-
-                $previousfn = '';
-                next unless exists $RFC822Head->{ $lhs };
-
-                $previousfn  = $lhs;
-                $rfc822part .= $e."\n";
-
-            } elsif( $e =~ m/\A[ \t]+/ ) {
-                # Continued line from the previous line
-                next if $rfc822next->{ $previousfn };
-                $rfc822part .= $e."\n" if exists $LongFields->{ $previousfn };
-
-            } else {
-                # Check the end of headers in rfc822 part
-                next unless exists $LongFields->{ $previousfn };
-                next if length $e;
-                $rfc822next->{ $previousfn } = 1;
+            unless( length $e ) {
+                $blanklines++;
+                last if $blanklines > 1;
+                next;
             }
+            push @$rfc822list, $e;
 
         } else {
             # Before "message/rfc822"
@@ -143,7 +123,7 @@ sub scan {
             # Original message follows.
             $v = $dscontents->[ -1 ];
 
-            if( $e =~ m/\A(.+)[ ](.+)[:][ \t]*([^ ]+[@][^ ]+)\z/ ) {
+            if( $e =~ m/\A(.+)[ ](.+)[:][ \t]*([^ ]+[@][^ ]+)/ ) {
                 # Unknown user: kijitora@example.com
                 if( length $v->{'recipient'} ) {
                     # There are multiple recipient addresses in the message body.
@@ -177,18 +157,10 @@ sub scan {
 
     return undef unless $recipients;
     require Sisimai::String;
-    require Sisimai::SMTP::Status;
 
     for my $e ( @$dscontents ) {
         $e->{'agent'} = __PACKAGE__->smtpagent;
 
-        if( scalar @{ $mhead->{'received'} } ) {
-            # Get localhost and remote host name from Received header.
-            my $r0 = $mhead->{'received'};
-            $e->{'lhost'} ||= shift @{ Sisimai::RFC5322->received( $r0->[0] ) };
-            $e->{'rhost'} ||= pop @{ Sisimai::RFC5322->received( $r0->[-1] ) };
-        }
-
         if( exists $e->{'alterrors'} && length $e->{'alterrors'} ) {
             # Copy alternative error message
             $e->{'diagnosis'} = $e->{'alterrors'}.' '.$e->{'diagnosis'};
@@ -210,13 +182,10 @@ sub scan {
             $e->{'reason'} = $r;
             last;
         }
-
-        $e->{'status'} = Sisimai::SMTP::Status->find( $e->{'diagnosis'} );
-        $e->{'spec'}   = $e->{'reason'} eq 'mailererror' ? 'X-UNIX' : 'SMTP';
-        $e->{'action'} = 'failed' if $e->{'status'} =~ m/\A[45]/;
     }
 
-    return { 'ds' => $dscontents, 'rfc822' => $rfc822part };
+    $rfc822part = Sisimai::RFC5322->weedout( $rfc822list );
+    return { 'ds' => $dscontents, 'rfc822' => $$rfc822part };
 }
 
 1;
