@@ -82,9 +82,8 @@ sub scan {
 
     require Sisimai::MTA;
     require Sisimai::Address;
-    require Sisimai::RFC5322;
 
-    my $dscontents = []; push @$dscontents, Sisimai::MTA->DELIVERYSTATUS;
+    my $dscontents = [ Sisimai::MTA->DELIVERYSTATUS ];
     my @hasdivided = split( "\n", $$mbody );
     my $recipients = 0;     # (Integer) The number of 'Final-Recipient' header
     my $maxmsgline = 5;     # (Integer) Max message length(lines)
@@ -136,12 +135,6 @@ sub scan {
     $v->{'date'}      = $mhead->{'date'};
     $v->{'status'}    = '';
 
-    if( scalar @{ $mhead->{'received'} } ) {
-        # Get localhost and remote host name from Received header.
-        my $r0 = $mhead->{'received'};
-        $v->{'lhost'} ||= shift @{ Sisimai::RFC5322->received( $r0->[0] ) };
-        $v->{'rhost'} ||= pop @{ Sisimai::RFC5322->received( $r0->[-1] ) };
-    }
     return { 'ds' => $dscontents, 'rfc822' => '' };
 }
 
